@@ -855,12 +855,12 @@ public:
      *   kdDebug(300) << "node isn't an element node" << endl;
      * </pre>
      */
-    bool isNull() const;
+    bool isNull() const { return impl == 0; }
 
     /**
      * @internal handle to the implementation object
      */
-    NodeImpl *handle() const;
+    NodeImpl *handle() const { return impl; }
 
     /**
      * @internal returns the index of a node
@@ -939,9 +939,12 @@ public:
      * @internal
      * not part of the DOM
      */
+    Node itemById (const DOMString& elementId) const;
+
     NodeListImpl *handle() const;
     bool isNull() const;
 
+
 protected:
     NodeList(const NodeListImpl *i);
     NodeListImpl *impl;
