@@ -1,4 +1,3 @@
-// -*- c-basic-offset: 2 -*-
 /*
  *  This file is part of the KDE libraries
  *  Copyright (C) 2001 Peter Kelly (pmk@post.com)
@@ -20,12 +19,14 @@
 
 #include "kjs_traversal.h"
 #include "kjs_traversal.lut.h"
+
 #include "kjs_proxy.h"
 #include <dom/dom_node.h>
 #include <xml/dom_nodeimpl.h>
 #include <xml/dom_docimpl.h>
 #include <khtmlview.h>
 #include <kdebug.h>
+#include <kjs/protect.h>
 
 using namespace KJS;
 
@@ -149,11 +150,15 @@ Value NodeFilterConstructor::getValueProperty(ExecState *, int token) const
   return Number(token);
 }
 
-Value KJS::getNodeFilterConstructor(ExecState *exec)
+namespace KJS {
+
+Value getNodeFilterConstructor(ExecState *exec)
 {
   return cacheGlobalObject<NodeFilterConstructor>(exec, "[[nodeFilter.constructor]]");
 }
 
+}
+
 // -------------------------------------------------------------------------
 
 const ClassInfo DOMNodeFilter::info = { "NodeFilter", 0, 0, 0 };
@@ -320,7 +325,9 @@ short JSNodeFilterCondition::acceptNode(const DOM::Node &node) const
 {
     KHTMLPart *part = static_cast<DOM::DocumentImpl *>(node.handle()->docPtr()->document())->part();
     KJSProxy *proxy = KJSProxy::proxy(part);
+
     if (proxy && filter.implementsCall()) {
+        InterpreterLock lock;
         ExecState *exec = proxy->interpreter()->globalExec();
         List args;
         args.append(getDOMNode(exec,node));
