@@ -617,28 +617,36 @@ void XMLHttpRequest::addToRequestsByDocument()
   requests->insert(this, this);
 }
 
-void XMLHttpRequest::removeFromRequestsByDocument()
+static void removeFromRequestsByDocument(XMLHttpRequest* request, DOM::DocumentImpl* doc)
 {
   assert(doc);
 
-  QPtrDict<XMLHttpRequest> *requests = requestsByDocument().find(doc);
+  QPtrDict<XMLHttpRequest> *requests = XMLHttpRequest::requestsByDocument().find(doc);
 
   // Since synchronous loads are not added to requestsByDocument(), we need to make sure we found the request.
-  if (!requests || !requests->find(this))
+  if (!requests || !requests->find(request))
     return;
 
-  requests->remove(this);
+  requests->remove(request);
 
   if (requests->isEmpty()) {
-    requestsByDocument().remove(doc);
+    XMLHttpRequest::requestsByDocument().remove(doc);
     delete requests;
   }
 }
 
+void XMLHttpRequest::removeFromRequestsByDocument()
+{
+  ::removeFromRequestsByDocument(this, doc);
+}
+
 void XMLHttpRequest::cancelRequests(DOM::DocumentImpl *d)
 {
-  while (QPtrDict<XMLHttpRequest> *requests = requestsByDocument().find(d))
-    QPtrDictIterator<XMLHttpRequest>(*requests).current()->abort();
+  while (QPtrDict<XMLHttpRequest> *requests = requestsByDocument().find(d)) {
+    XMLHttpRequest* request = QPtrDictIterator<XMLHttpRequest>(*requests).current();
+    ::removeFromRequestsByDocument(request, d);
+    request->abort();
+  }
 }
 
 Value XMLHttpRequestProtoFunc::tryCall(ExecState *exec, Object &thisObj, const List &args)
