@@ -25,9 +25,15 @@
 #include "xml/dom_docimpl.h"
 #include "xml/dom_stringimpl.h"
 #include "css/css_stylesheetimpl.h"
+#ifdef KHTML_XSLT
+#include "xsl_stylesheetimpl.h"
+#endif
 #include "misc/loader.h"
+#include "xml/xml_tokenizer.h"
 
-using namespace DOM;
+using khtml::parseAttributes;
+
+namespace DOM {
 
 EntityImpl::EntityImpl(DocumentPtr *doc) : NodeBaseImpl(doc)
 {
@@ -299,6 +305,9 @@ ProcessingInstructionImpl::ProcessingInstructionImpl(DocumentPtr *doc) : NodeBas
     m_sheet = 0;
     m_cachedSheet = 0;
     m_loading = false;
+#ifdef KHTML_XSLT
+    m_isXSL = false;
+#endif
 }
 
 ProcessingInstructionImpl::ProcessingInstructionImpl(DocumentPtr *doc, DOMString _target, DOMString _data) : NodeBaseImpl(doc)
@@ -312,6 +321,9 @@ ProcessingInstructionImpl::ProcessingInstructionImpl(DocumentPtr *doc, DOMString
     m_sheet = 0;
     m_cachedSheet = 0;
     m_localHref = 0;
+#ifdef KHTML_XSLT
+    m_isXSL = false;
+#endif
 }
 
 ProcessingInstructionImpl::~ProcessingInstructionImpl()
@@ -389,32 +401,67 @@ bool ProcessingInstructionImpl::childTypeAllowed( unsigned short /*type*/ )
     return false;
 }
 
-void ProcessingInstructionImpl::checkStyleSheet()
+bool ProcessingInstructionImpl::checkStyleSheet()
 {
     if (m_target && DOMString(m_target) == "xml-stylesheet") {
         // see http://www.w3.org/TR/xml-stylesheet/
         // ### check that this occurs only in the prolog
         // ### support stylesheet included in a fragment of this (or another) document
         // ### make sure this gets called when adding from javascript
-        XMLAttributeReader attrReader(DOMString(m_data).string());
         bool attrsOk;
-        QXmlAttributes attrs = attrReader.readAttrs(attrsOk);
+        const QMap<QString, QString> attrs = parseAttributes(m_data, attrsOk);
         if (!attrsOk)
-            return;
-        if (attrs.value("type") != "text/css")
-            return;
+            return true;
+        QMap<QString, QString>::ConstIterator i = attrs.find("type");
+        QString type;
+        if (i != attrs.end())
+            type = *i;
+        
+        bool isCSS = type.isEmpty() || type == "text/css";
+#ifdef KHTML_XSLT
+        m_isXSL = (type == "text/xml" || type == "text/xsl" || type == "application/xml" ||
+                   type == "application/xhtml+xml" || type == "application/rss+xml" || type == "application/atom=xml");
+        if (!isCSS && !m_isXSL)
+#else
+        if (!isCSS)
+#endif
+            return true;
 
-        DOMString href = attrs.value("href");
+#ifdef KHTML_XSLT
+        if (m_isXSL)
+            getDocument()->tokenizer()->setTransformSource(getDocument());
+#endif
+
+        i = attrs.find("href");
+        QString href;
+        if (i != attrs.end())
+            href = *i;
 
         if (href.length()>1)
         {
             if (href[0]=='#')
             {
+                DOMString newLocalHref = href.mid(1);
                 if (m_localHref)
                     m_localHref->deref();
-                m_localHref=href.implementation()->split(1);
+                m_localHref = newLocalHref.implementation();
                 if (m_localHref)
                     m_localHref->ref();
+#ifdef KHTML_XSLT
+                // We need to make a synthetic XSLStyleSheetImpl that is embedded.  It needs to be able
+                // to kick off import/include loads that can hang off some parent sheet.
+                if (m_isXSL) {
+                    if (m_sheet)
+                        m_sheet->deref();
+                    XSLStyleSheetImpl* localSheet = new XSLStyleSheetImpl(this, m_localHref, true);
+                    localSheet->setDocument((xmlDocPtr)getDocument()->transformSource());
+                    localSheet->ref();
+                    localSheet->loadChildSheets();
+                    m_sheet = localSheet;
+                    m_loading = false;
+                }                    
+                return !m_isXSL;
+#endif
             }
             else
             {
@@ -424,26 +471,38 @@ void ProcessingInstructionImpl::checkStyleSheet()
 		    m_loading = true;
 		    getDocument()->addPendingSheet();
 		    if (m_cachedSheet) m_cachedSheet->deref(this);
-		    m_cachedSheet = getDocument()->docLoader()->requestStyleSheet(getDocument()->completeURL(href.string()), QString::null);
+#ifdef KHTML_XSLT
+                    if (m_isXSL)
+                        m_cachedSheet = getDocument()->docLoader()->requestXSLStyleSheet(getDocument()->completeURL(href));
+                    else
+#endif
+                    m_cachedSheet = getDocument()->docLoader()->requestStyleSheet(getDocument()->completeURL(href), QString::null);
 		    if (m_cachedSheet)
 			m_cachedSheet->ref( this );
+#ifdef KHTML_XSLT
+                    return !m_isXSL;
+#endif
 		}
             }
 
         }
     }
+    
+    return true;
 }
 
-StyleSheetImpl *ProcessingInstructionImpl::sheet() const
+StyleSheetImpl* ProcessingInstructionImpl::sheet() const
 {
     return m_sheet;
 }
 
 bool ProcessingInstructionImpl::isLoading() const
 {
-    if(m_loading) return true;
-    if(!m_sheet) return false;
-    return static_cast<CSSStyleSheetImpl *>(m_sheet)->isLoading();
+    if (m_loading)
+        return true;
+    if (!m_sheet)
+        return false;
+    return m_sheet->isLoading();
 }
 
 void ProcessingInstructionImpl::sheetLoaded()
@@ -452,11 +511,16 @@ void ProcessingInstructionImpl::sheetLoaded()
         getDocument()->stylesheetLoaded();
 }
 
-void ProcessingInstructionImpl::setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheet)
+void ProcessingInstructionImpl::setStyleSheet(const DOMString &url, const DOMString &sheet)
 {
     if (m_sheet)
 	m_sheet->deref();
-    m_sheet = new CSSStyleSheetImpl(getDocument(), url);
+#ifdef KHTML_XSLT
+    if (m_isXSL)
+        m_sheet = new XSLStyleSheetImpl(this, url);
+    else
+#endif
+        m_sheet = new CSSStyleSheetImpl(this, url);
     m_sheet->ref();
     m_sheet->parseString(sheet);
     if (m_cachedSheet)
@@ -489,35 +553,4 @@ DOMString ProcessingInstructionImpl::toString() const
     return result;
 }
 
-// -------------------------------------------------------------------------
-
-XMLAttributeReader::XMLAttributeReader(QString _attrString)
-{
-    m_attrString = _attrString;
-}
-
-XMLAttributeReader::~XMLAttributeReader()
-{
-}
-
-QXmlAttributes XMLAttributeReader::readAttrs(bool &ok)
-{
-    // parse xml file
-    QXmlInputSource source;
-    source.setData("<?xml version=\"1.0\"?><attrs "+m_attrString+" />");
-    QXmlSimpleReader reader;
-    reader.setContentHandler( this );
-    ok = reader.parse( source );
-    return attrs;
-}
-
-bool XMLAttributeReader::startElement(const QString& /*namespaceURI*/, const QString& localName,
-                                      const QString& /*qName*/, const QXmlAttributes& atts)
-{
-    if (localName == "attrs") {
-        attrs = atts;
-        return true;
-    }
-    else
-        return false; // we shouldn't have any other elements
-}
+} // namespace
