@@ -363,6 +363,7 @@ typedef enum
 
 - (int)numPendingOrLoadingRequests;
 - (BOOL)doneProcessingData;
+- (BOOL)shouldClose;
 
 - (void)setDrawsBackground:(BOOL)drawsBackround;
 
@@ -515,6 +516,8 @@ typedef enum
 - (void)runJavaScriptAlertPanelWithMessage:(NSString *)message;
 - (BOOL)runJavaScriptConfirmPanelWithMessage:(NSString *)message;
 - (BOOL)runJavaScriptTextInputPanelWithPrompt:(NSString *)prompt defaultText:(NSString *)defaultText returningText:(NSString **)result;
+- (BOOL)canRunBeforeUnloadConfirmPanel;
+- (BOOL)runBeforeUnloadConfirmPanelWithMessage:(NSString *)message;
 - (void)addMessageToConsole:(NSDictionary *)message;
 
 - (id <WebCoreResourceHandle>)startLoadingResource:(id <WebCoreResourceLoader>)loader withURL:(NSURL *)URL customHeaders:(NSDictionary *)customHeaders;
